-------------------------------------------------------
--! @file addsub4.vhd
--! @brief 4-bit Adder/Subtractor
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-07
-------------------------------------------------------

entity addsub4 is
  port (
    op :  in bit; -- 0 is add, 1 is sub
    x :   in bit_vector (3 downto 0);
    y :   in bit_vector (3 downto 0);
    r :   out bit_vector (3 downto 0); -- result: x+y or x-y
    ovf : out bit -- Overflow
    );
end entity addsub4;

architecture addsub4_arch of addsub4 is

component adder4 is
  port (
    cin :  in bit;
    x :    in bit_vector (3 downto 0);
    y :    in bit_vector (3 downto 0);
    sum :  out bit_vector (3 downto 0);
    cout : out bit
    );
end component adder4;

signal a,b,b_l,c,result : bit_vector (3 downto 0);

begin
UADD : component adder4 port map (op,a,c,result,open);

a <= x;
b <= y;
b_l <= not(b);
r <= result;

with op select
  c <=
  b when '0',
  b_l when '1',
  "XXXX" when others;
  
ovf <= not((a(3) xor c(3))) and (a(3) xor result(3));
                                  
end architecture addsub4_arch;
